<?php

namespace Drupal\acquia_contenthub\Commands;

use Acquia\ContentHubClient\ContentHubClient;
use Acquia\ContentHubClient\Settings;
use Consolidation\OutputFormatters\StructuredData\RowsOfFields;
use Drupal\acquia_contenthub\Client\ClientFactory;
use Drupal\acquia_contenthub\EntityCdfSerializer;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\depcalc\DependencyCalculator;
use Drupal\depcalc\DependencyStack;
use Drupal\depcalc\DependentEntityWrapper;
use Drush\Commands\DrushCommands;
use Drupal\Component\Uuid\Uuid;
use Drupal\Core\Form\FormState;
use Drupal\Core\Url;
use Drush\Log\LogLevel;

class AcquiaContentHubCommands extends DrushCommands {

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface 
   */
  protected $configFactory;

  /**
   * The client factory.
   *
   * @var \Drupal\acquia_contenthub\Client\ClientFactory 
   */
  protected $clientFactory;

  /**
   * The dependency calculator.
   *
   * @var \Drupal\depcalc\DependencyCalculator
   */
  protected $calculator;

  /**
   * The CDF Serializer.
   *
   * @var \Drupal\acquia_contenthub\EntityCdfSerializer
   */
  protected $serializer;

  /**
   * AcquiaContentHubCommands constructor.
   *
   * @param DependencyCalculator $calculator
   *   The dependency calculator.
   * @param EntityCdfSerializer $serializer
   *   The dependency calculator.
   * @param ConfigFactoryInterface $config_factory
   *   The configuration factory.
   * @param \Drupal\acquia_contenthub\Client\ClientFactory $client_factory
   *   The client factory.
   */
  public function __construct(DependencyCalculator $calculator, EntityCdfSerializer $serializer, ConfigFactoryInterface $config_factory, ClientFactory $client_factory) {
    $this->configFactory = $config_factory;
    $this->clientFactory = $client_factory;
    $this->calculator = $calculator;
    $this->serializer = $serializer;
  }

  /**
   * Retrieves an Entity from a local source or contenthub
   *
   * @param string $op
   *   The operation being performed
   * @param string $uuid
   *   Entity identifier or entity's UUID
   * @param string $entity_type
   *   The entity type in case of local retrieval
   *
   * @command acquia:contenthub-entity
   * @aliases ach-ent
   *
   * @throws \Exception
   */
  public function contenthubEntity($op, $uuid, $entity_type = NULL) {
    $client = $this->clientFactory->getClient();

    if (empty($uuid)) {
      throw new \Exception("Please supply the uuid of the entity you want to retrieve.");
    }

    switch ($op) {
      case 'local':
        if (empty($entity_type)) {
          throw new \Exception(dt("Entity_type is required for local entities"));
        }
        $repository = \Drupal::service('entity.repository');
        $entity = $repository->loadEntityByUuid($entity_type, $uuid);

        $wrapper = new DependentEntityWrapper($entity);
        $stack = new DependencyStack();
        $this->calculator->calculateDependencies($wrapper, $stack);
        $entities = NestedArray::mergeDeep(
          [$wrapper->getEntity()->uuid() => $wrapper],
          $stack->getDependenciesByUuid(array_keys($wrapper->getDependencies()))
        );
        $objects = $this->serializer->serializeEntities(...array_values($entities));
        $data = [];
        foreach ($objects as $object) {
          $data[$object->getUuid()] = $object->toArray();
        }
        $json = json_encode(
          $data,
          JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES
        );
        $this->output()->writeln($json);
        break;

      case 'remote':
        $json = json_encode(
          $client->getEntity($uuid)->toArray(),
          JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE | JSON_UNESCAPED_SLASHES
        );
        $this->output()->writeln($json);
        break;

      default:
        // Invalid operation.
        throw new \Exception(dt('The op "@op" is invalid', ['@op' => $op]));
    }
  }

  /**
   * Prints the CDF from a local source (drupal site)
   *
   * @param string $entity_type
   *   The entity type to load.
   * @param string $entity_id
   *   The entity identifier or entity's UUID.
   *
   * @return void
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   *
   * @command acquia:contenthub-local
   * @aliases ach-lo,acquia-contenthub-local
   *
   * @deprecated
   */
  public function contenthubLocal($entity_type, $entity_id) {
    $entity_type_manager = \Drupal::entityTypeManager();

    /** @var \Drupal\Core\Entity\EntityRepository $entity_repository */
    $entity_repository = \Drupal::service('entity.repository');

    if (empty($entity_type) || empty($entity_id)) {
      throw new \Exception(dt("Missing required parameters: entity_type and entity_id (or entity's uuid)"));
    }
    elseif (!$entity_type_manager->getDefinition($entity_type)) {
      throw new \Exception(dt("Entity type @entity_type does not exist", [
        '@entity_type' => $entity_type,
      ]));
    }
    else {
      if (Uuid::isValid($entity_id)) {
        $entity = $entity_repository->loadEntityByUuid($entity_type, $entity_id);
      }
      else {
        $entity = $entity_type_manager->getStorage($entity_type)->load($entity_id);
      }
    }
    if (!$entity) {
      $this->output()->writeln(dt("Entity having entity_type = @entity_type and entity_id = @entity_id does not exist.", [
        '@entity_type' => $entity_type,
        '@entity_id' => $entity_id,
      ]));
    }
    // If nothing else, return our object structure.
    $this->contenthubEntity('local', $entity->uuid(), $entity_type);
  }

  /**
   * Prints the CDF from a remote source (Content Hub)
   *
   * @param string $uuid
   *   The entity's UUID.
   *
   * @command acquia:contenthub-remote
   * @aliases ach-re,acquia-contenthub-remote
   *
   * @throws \Exception
   */
  public function contenthubRemote($uuid) {
    if (FALSE === Uuid::isValid($uuid)) {
      throw new \Exception(dt("Argument provided is not a UUID."));
    }

    $this->contenthubEntity('remote', $uuid);
  }

  /**
   * List entities from the Content Hub using the listEntities() method.
   *
   * @param array $options An associative array of options whose values come
   *   from cli, aliases, config, etc.
   *
   * @option limit
   *   The number of entities to be listed
   * @option start
   *   The offset to start listing the entities (Useful for pagination).
   * @option origin
   *   The Client's Origin UUID.
   * @option language
   *   The Language that will be used to filter field values.
   * @option attributes
   *   The attributes to display for all listed entities
   * @option type
   *   The entity type
   * @option filters
   *   Filters entities according to a set of of conditions as a key=value pair
   *   separated by commas. You could use regex too.
   *
   * @command acquia:contenthub-list
   * @aliases ach-list,acquia-contenthub-list
   * @return mixed
   * @throws \Exception
   */
  public function contenthubList(array $options = ['limit' => null, 'start' => null, 'origin' => null, 'language' => null, 'attributes' => null, 'type' => null, 'filters' => null]) {
    $client = $this->clientFactory->getClient();
    if (!$client) {
      throw new \Exception(dt('Error trying to connect to the Content Hub. Make sure this site is registered to Content hub.'));
    }
    $list_options = [];

    // Obtaining the limit.
    $limit = $options['limit'];
    if (isset($limit)) {
      $limit = (int) $limit;
      if ($limit < 1 || $limit > 1000) {
        throw new \Exception(dt("The limit has to be an integer from 1 to 1000."));
      }
      else {
        $list_options['limit'] = $limit;
      }
    }

    // Obtaining the offset.
    $start = $options['start'];
    if (isset($start)) {
      if (!is_numeric($start)) {
        throw new \Exception(dt("The start offset has to be numeric starting from 0."));
      }
      else {
        $list_options['start'] = $start;
      }
    }

    // Filtering by origin.
    $origin = $options['origin'];
    if (isset($origin)) {
      if (Uuid::isValid($origin)) {
        $list_options['origin'] = $origin;
      }
      else {
        throw new \Exception(dt("The origin has to be a valid UUID."));
      }
    }

    // Filtering by language.
    // TODO: Add a query to validate languages in plexus
    $language = $options['language'];
    if (isset($language)) {
      $list_options['language'] = $language;
    }

    // Filtering by fields.
    $fields = $options['attributes'];
    if (isset($fields)) {
      $list_options['fields'] = $fields;
    }

    // Filtering by type.
    $type = $options['type'];
    if (isset($type)) {
      $list_options['type'] = $type;
    }

    // Building the filters.
    $filters = $options['filters'];
    if (isset($filters)) {
      $filters = isset($filters) ? explode(",", $filters) : FALSE;
      foreach ($filters as $key => $filter) {
        list($name, $value) = explode("=", $filter);
        $filters[$name] = $value;
        unset($filters[$key]);
      }
      $list_options['filters'] = $filters;
    }

    $list = $client->listEntities($list_options);
    $this->output()->writeln(print_r($list, TRUE));
  }

  /**
   * Deletes a single entity from the Content Hub
   *
   * @param string $uuid
   *   The entity's UUID.
   *
   * @command acquia:contenthub-delete
   * @aliases ach-del,acquia-contenthub-delete
   * @throws \Exception
   */
  public function contenthubDelete($uuid) {
    $client = $this->clientFactory->getClient();

    if (!Uuid::isValid($uuid)) {
      throw new \Exception(dt("Argument provided is not a UUID."));
    }
    $warning_message = dt('Are you sure you want to delete the entity with uuid = @uuid from the Content Hub? There is no way back from this action!', [
      '@uuid' => $uuid,
    ]);
    if ($this->io()->confirm($warning_message)) {
      if ($client->deleteEntity($uuid)) {
        $this->output()->writeln(dt("Entity with UUID = @uuid has been successfully deleted from the Content Hub.", [
          '@uuid' => $uuid,
        ]));
      }
      else {
        throw new \Exception(dt("Entity with UUID = @uuid cannot be deleted.", [
          '@uuid' => $uuid,
        ]));
      }
    }
  }

  /**
   * Purges all entities from Acquia Content Hub. WARNING! Be VERY careful when
   * using this command. This destructive command requires elevated keys. Every
   * subsequent execution of this command will override the backup created
   * by the previous call.
   *
   * @param string $api
   *   The API key.
   * @param string $secret
   *   The secret key.
   *
   * @command acquia:contenthub-purge
   * @aliases ach-purge,acquia-contenthub-purge
   * @throws \Exception
   */
  public function contenthubPurge($api = NULL, $secret = NULL) {
    $warning_message = "Are you sure you want to PURGE your Content Hub Subscription?\n" .
      "*************************************************************************************\n" .
      "PROCEED WITH CAUTION. THIS ACTION WILL PURGE ALL EXISTING ENTITIES IN YOUR CONTENT HUB SUBSCRIPTION.\n" .
      "While a backup is created for use by the restore command, restoration may not be timely and is not guaranteed. Concurrent or frequent\n" .
      "use of this command may result in an inability to restore. You can always republish your content as a means of 'recovery'.
    For more information, check https://docs.acquia.com/content-hub.\n" .
      "*************************************************************************************\n" .
      "Are you sure you want to proceed?\n";
    if ($this->io()->confirm($warning_message)) {
      if (!empty($api) && !empty($secret)) {
        $client = $this->resetConnection($this->clientFactory->getClient(), $api, $secret);
      }
      else {
        $client = $this->clientFactory->getClient();
      }
      if (!$client) {
        throw new \Exception(dt('Error trying to connect to the Content Hub. Make sure this site is registered to Content hub.'));
      }

      // Execute the 'purge' command.
      $response = $client->purge();

      if (isset($response['success']) && $response['success'] === TRUE) {
        // Deleting exported entities from the Tracking Table.
        /** @var \Drupal\acquia_contenthub\ContentHubEntitiesTracking $entities_tracking */
        \Drupal::database()->truncate('acquia_contenthub_publisher_export_tracking')->execute();
        $this->output()->writeln("Your Subscription is being purged. All clients who have registered to received webhooks will be notified with a reindex webhook when the purge process has been completed.\n");
      }
      else {
        throw new \Exception(dt("Error trying to purge your subscription. You might require elevated keys to perform this operation."));
      }
    }
  }

  /**
   * Restores the backup taken by a previous execution of the "purge" command.
   * WARNING! Be VERY careful when using this command. This destructive command
   * requires elevated keys. By restoring a backup you will delete all the
   * existing entities in your subscription.
   *
   * @param string $api
   *   The API key.
   * @param $secret
   *   The (optional) string secret key.
   *
   * @command acquia:contenthub-restore
   * @aliases ach-restore,acquia-contenthub-restore
   * @throws \Exception
   */
  public function contenthubRestore($api, $secret) {
    $warning_message = "Are you sure you want to RESTORE the latest backup taken after purging your Content Hub Subscription?\n" .
      "*************************************************************************************\n" .
      "PROCEED WITH CAUTION. THIS ACTION WILL ELIMINATE ALL EXISTING ENTITIES IN YOUR CONTENT HUB SUBSCRIPTION.\n" .
      "This restore command should only be used after an accidental purge event has taken place *and* completed. This will attempt to restore\n" .
      "from the last purge-generated backup. In the event this fails, you will need to republish your content to Content Hub.
    For more information, check https://docs.acquia.com/content-hub.\n" .
      "*************************************************************************************\n" .
      "Are you sure you want to proceed?\n";
    if ($this->io()->confirm($warning_message)) {
      if (!empty($api) && !empty($secret)) {
        $client = $this->resetConnection($this->clientFactory->getClient(), $api, $secret);
      }
      else {
        $client = $this->clientFactory->getClient();
      }

      // Execute the 'restore' command.
      if (!$client) {
        throw new \Exception(dt('Error trying to connect to the Content Hub. Make sure this site is registered to Content hub.'));
      }
      $response = $client->restore();

      if (isset($response['success']) && $response['success'] === TRUE) {
        $this->output()->writeln("Your Subscription is being restored. All clients who have registered to received webhooks will be notified with a reindex webhook when the restore process has been completed.\n");
      }
      else {
        throw new \Exception(dt("Error trying to restore your subscription from a backup copy. You might require elevated keys to perform this operation."));
      }
    }
  }

  /**
   * View Historic entity logs from Content Hub
   *
   * @param string $api
   *   API Key
   * @param string $secret
   *   Secret Key
   * @param array $options
   *
   * @throws \Exception
   * @internal param array $request_options An associative array of options
   *   whose values come from cli, aliases, config, etc.
   *
   * @option query
   *   The Elastic Search Query to search for logs.
   * @option size
   *   The number of log entries to be listed.
   * @option from
   *   The offset to start listing the log entries (Useful for pagination).
   *
   * @command acquia:contenthub-logs
   * @field-labels
   *   timestamp: Timestamp
   *   type: Type
   *   client: Client ID
   *   entity_uuid: Entity UUID
   *   status: Status
   *   request_id: Request ID
   *   id: ID
   * @aliases ach-logs,acquia-contenthub-logs
   * @return \Consolidation\OutputFormatters\StructuredData\RowsOfFields
   */
  public function contenthubLogs($api, $secret, array $options = ['query' => null, 'size' => null, 'from' => null]) {
    if (!empty($api) && !empty($secret)) {
      $client = $this->resetConnection($this->clientFactory->getClient(), $api, $secret);
    }
    else {
      $client = $this->clientFactory->getClient();
    }

    $request_options = [];
    // Obtaining the limit.
    $size = $options["size"];
    if (isset($size)) {
      $size = (int) $size;
      if ($size < 1 || $size > 1000) {
        throw new \Exception(dt("The size has to be an integer from 1 to 1000."));
      }
      else {
        $request_options['size'] = $size;
      }
    }

    // Obtaining the offset.
    $from = $options["from"];
    if (isset($from)) {
      if (!is_numeric($from)) {
        throw new \Exception(dt("The start offset has to be numeric starting from 0."));
      }
      else {
        $request_options['from'] = $from;
      }
    }

    // Obtaining the query.
    $query = $options["query"];
    $query = !empty($query) ? $query : '';

    // Execute the 'history' command.
    if (!$client) {
      throw new \Exception(dt('Error trying to connect to the Content Hub. Make sure this site is registered to Content hub.'));
    }
    $logs = $client->logs($query, $request_options);
    if ($logs) {
      $rows = [];
      if (isset($logs['hits']['hits'])) {
        foreach ($logs['hits']['hits'] as $log) {
          $rows[] = [
            'timestamp' => $log['_source']['timestamp'],
            'type' => strtoupper($log['_source']['type']),
            'client' => $log['_source']['client'],
            'entity_uuid' => $log['_source']['entity'],
            'status' => strtoupper($log['_source']['status']),
            'request_id' => $log['_source']['request_id'],
            'id' => $log['_source']['id'],
          ];
        }
      }

      // Sort results DESC by 'timestamp' before presenting.
      usort($rows, function ($a, $b) {
        return strcmp($b["timestamp"], $a["timestamp"]);
      });
      return new RowsOfFields($rows);
    }
    else {
      throw new \Exception(dt("Error trying to print the entity logs."));
    }
  }

  /**
   * Shows Elastic Search field mappings from Content Hub
   *
   * @command acquia:contenthub-mapping
   * @aliases ach-mapping,acquia-contenthub-mapping
   */
  public function contenthubMapping() {
    $client = $this->clientFactory->getClient();

    if (!$client) {
      throw new \Exception(dt('Error trying to connect to the Content Hub. Make sure this site is registered to Content hub.'));
    }
    $output = $client->mapping();

    if ($output) {
      $this->output()->writeln(print_r($output, TRUE));
    }
    else {
      throw new \Exception(dt("Error trying to print the elastic search field mappings."));
    }
  }

  /**
   * Regenerates the Shared Secret used for Webhook Verification
   *
   * @command acquia:contenthub-regenerate-secret
   * @aliases ach-regsec,acquia-contenthub-regenerate-secret
   */
  public function contenthubRegenerateSecret() {
    $client = $this->clientFactory->getClient();
    $warning_message = "Are you sure you want to REGENERATE your shared-secret in the Content Hub?\n" .
      "*************************************************************************************\n" .
      "PROCEED WITH CAUTION. THIS COULD POTENTIALLY LEAD TO HAVING SOME SITES OUT OF SYNC.\n" .
      "Make sure you have ALL your sites correctly configured to receive webhooks before attempting to do this.\n" .
      "For more information, check https://docs.acquia.com/content-hub/known-issues.\n" .
      "*************************************************************************************\n";
    if ($this->io()->confirm($warning_message)) {
      if (!$client) {
        throw new \Exception(dt('Error trying to connect to the Content Hub. Make sure this site is registered to Content hub.'));
      }
      $output = $client->regenerateSharedSecret();

      if ($output) {
        $this->output()->writeln("Your Shared Secret has been regenerated. All clients who have registered to received webhooks are being notified of this change.\n");
      }
      else {
        throw new \Exception(dt("Error trying to regenerate the shared-secret in your subscription. Try again later."));
      }
    }
  }

  /**
   * Updates the Shared Secret used for Webhook Verification
   *
   * @command acquia:contenthub-update-secret
   * @aliases ach-upsec,acquia-contenthub-update-secret
   */
  public function contenthubUpdateSecret() {
    $client = $this->clientFactory->getClient();

    if (!$client) {
      throw new \Exception(dt('The Content Hub client is not connected so the shared secret can not be updated.'));
    }

    $remote = $client->getRemoteSettings();
    $provider = $this->clientFactory->getProvider();
    if (!empty($remote['shared_secret']) && $provider === 'core_config') {
      $config = $this->configFactory->getEditable('acquia_contenthub.admin_settings');
      $config->set('shared_secret', $remote['shared_secret']);
      $config->save();
      $this->output()->writeln(dt('The shared secret has been updated to: @secret', ['@secret' => $remote['shared_secret']]));
      return;
    }
    $this->output()->writeln(dt('The settings object is read only. Your remote shared secret is: @secret Please update your settings object if necessary.', ['@secret' => $remote['shared_secret']]));
  }

  /**
   * Connects a site with contenthub.
   *
   * @param array $options
   *
   * @command acquia:contenthub-connect-site
   * @aliases ach-connect,acquia-contenthub-connect-site
   *
   * @option hostname
   *   Content Hub API URL.
   * @option api_key
   *   Content Hub API Key.
   * @option secret
   *   Content Hub API Secret.
   * @option client_name
   *   The client name for this site.
   */
  public function contenthubConnectSite(array $options = ['hostname' => null, 'api_key' => null, 'secret_key' => null, 'client_name' => null]) {
    $uuid_service = \Drupal::service('uuid');

    // TODO: Revisit initial connection logic with our event subscibers.
    $settings = $this->clientFactory->getSettings();
    $config_origin = $settings->getUuid();

    $provider = $this->clientFactory->getProvider();
    $disabled = $provider != 'core_config';
    if ($disabled) {
      $this->logger()->log(LogLevel::CANCEL, dt("Settings are being provided by %provider, and already connected.", ['%provider' => $provider]));
      return;
    }

    if (!empty($config_origin)) {
      $this->logger()->log(LogLevel::CANCEL, dt('Site is already connected to Content Hub. Skipping.'));
      return;
    }

    $hostname = !empty($options['hostname']) ? $options['hostname'] : $this->io()->ask(dt('What is the Content Hub API URL?'), 'https://us-east-1.content-hub.acquia.com');
    $api_key = !empty($options['api_key']) ? $options['api_key'] : $this->io()->ask(dt('What is your Content Hub API Key?'));
    $secret = !empty($options['secret_key']) ? $options['secret_key'] : $this->io()->ask(dt('What is your Content Hub API Secret?'));
    $client_uuid = $uuid_service->generate();
    $client_name = !empty($options['client_name']) ? $options['client_name'] : $this->io()->ask(dt('What is the client name for this site?'), $client_uuid);

    $form_state = new FormState();
    $values['api_key'] = $api_key;
    $values['secret_key'] = $secret;
    $values['hostname'] = $hostname;
    $values['client_name'] = $client_name . '_' . $client_uuid;
    $values['op'] = t('Save configuration');
    $form_state->setValues($values);
    \Drupal::formBuilder()->submitForm('Drupal\acquia_contenthub\Form\ContentHubSettingsForm', $form_state);
  }

  /**
   * Disconnects a site with contenthub.
   *
   * @command acquia:contenthub-disconnect-site
   * @aliases ach-disconnect,acquia-contenthub-disconnect-site
   */
  public function contenthubDisconnectSite() {
    $client = $this->clientFactory->getClient();

    $provider = $this->clientFactory->getProvider();
    $disabled = $provider != 'core_config';
    if ($disabled) {
      $this->logger()->log(LogLevel::CANCEL, dt("Settings are being provided by %provider and cannot be disconnected manually.", ['%provider' => $provider]));
      return;
    }

    try {
      $client->deleteClient();
    }
    catch (\Exception $e) {
      $deleted = true;
    }

    $config_factory = \Drupal::configFactory();
    $config = $config_factory->getEditable('acquia_contenthub.admin_settings');
    $client_name = $config->get('client_name');
    $config->delete();

    // TODO: We should disconnect the webhook, but first we need to know its ours.
    $this->logger()->log(LogLevel::SUCCESS, dt("Successfully disconnected site %site from contenthub", ['%site' => $client_name]));
  }

  /**
   * Perform a webhook management operation.
   *
   * @param $op
   *   The operation to use. Options are: register, unregister, list.
   * @param array $options An associative array of options whose values come from cli, aliases, config, etc.
   *
   * @throws \Exception
   * @option webhook_url
   *   The webhook URL to register or unregister.
   *
   * @command acquia:contenthub-webhooks
   * @aliases ach-wh,acquia-contenthub-webhooks
   */
  public function contenthubWebhooks($op, array $options = ['webhook_url' => null]) {
    $client = $this->clientFactory->getClient();
    if (!$client) {
      throw new \Exception(dt('The Content Hub client is not connected so the webhook operations could not be performed.'));
    }

    $webhook_url = $options['webhook_url'];
    if (empty($webhook_url)) {
      $webhook_url = Url::fromUri('internal:/acquia-contenthub/webhook', ['absolute' => TRUE])->toString();
    }

    switch ($op) {
      case 'register':
        $success = $client->addWebhook($webhook_url);
        if (!$success) {
          $this->logger()->log(LogLevel::CANCEL, dt('Registering webhooks encountered an error.'));
        }
        else {
          $this->logger()->log(LogLevel::SUCCESS, dt('Registered Content Hub Webhook: @url | @uuid', ['@url' => $webhook_url, '@uuid' => $success['uuid']]));
        }
        break;

      case 'unregister':
        $webhook_url = isset($options['webhook_url']) ? $options['webhook_url'] : $webhook_url;
        $webhook = $client->getWebHook($webhook_url);
        if ($webhook) {
          $success = $client->deleteWebhook($webhook['uuid']);
        }
        if (!$success) {
          $this->logger()->log(LogLevel::CANCEL, dt('There was an error unregistering the URL: @url', ['@url' => $webhook_url]));
        }
        else {
          $this->logger()->log(LogLevel::SUCCESS, dt('Successfully unregistered Content Hub Webhook: @url', ['@url' => $webhook_url]));
        }
        break;

      case 'list':
        $webhooks = $client->getWebHooks();
        foreach ($webhooks as $index => $webhook) {
          $this->output()->writeln($index + 1 . '. ' . $webhook['url'] . ' - ' . $webhook['uuid']);
        }
        break;

      default:
        // Invalid operation.
        throw new \Exception(dt('The op "@op" is invalid', ['@op' => $op]));
    }
  }

  /**
   * Resets a connection to the client, allowing drush to connect to CH via a
   * different set of api keys/secrets.
   *
   * @param \Acquia\ContentHubClient\ContentHubClient $client
   * @param $api_key
   * @param $secret_key
   *
   * @return \Acquia\ContentHubClient\ContentHubClient
   */
  protected function resetConnection(ContentHubClient $client, $api_key, $secret_key) {
    $settings = $client->getSettings();
    $new_settings = new Settings($settings->getName(), $settings->getUuid(), $api_key, $secret_key, $settings->getUrl());
    // Find out the module version in use.
    $module_info = system_get_info('module', 'acquia_contenthub');
    $module_version = (isset($module_info['version'])) ? $module_info['version'] : '0.0.0';
    $drupal_version = (isset($module_info['core'])) ? $module_info['core'] : '0.0.0';
    $client_user_agent = 'AcquiaContentHub/' . $drupal_version . '-' . $module_version;

    // Override configuration.
    $config = [
      'base_url' => $settings->getUrl(),
      'client-user-agent' => $client_user_agent,
    ];

    return new ContentHubClient($config, $this->logger(), $new_settings, $new_settings->getMiddleware());
  }
}
