<?php

namespace Drupal\Tests\acquia_contenthub\Kernel;

use Acquia\ContentHubClient\CDFDocument;
use Acquia\ContentHubClient\Event\GetCDFTypeEvent;
use Acquia\ContentHubClient\EventSubscriber\DefaultCDF;
use Drupal\Component\Serialization\Json;
use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\depcalc\DependencyStack;
use Drupal\depcalc\DependentEntityWrapper;
use Drupal\KernelTests\Core\Entity\EntityKernelTestBase;

/**
 * Class ImportExportTestBase.
 *
 * @package Drupal\Tests\acquia_contenthub\Kernel
 */
class ImportExportTestBase extends EntityKernelTestBase {

  const ENTITY_REFERENCE_TYPES = [
    'file',
    'entity_reference',
    'entity_reference_revisions',
  ];

  const ENTITY_REFERENCE_IMAGE_TYPE = 'image';

  protected $fixtures = [];

  /**
   * {@inheritdoc}
   */
  public static $modules = [
    'system',
    'depcalc',
    'acquia_contenthub',
  ];

  /**
   * Returns fixture content.
   *
   * @param int $delta
   *   Fixture delta.
   *
   * @return false|string
   *   Fixture string if file exists.
   *
   * @throws \Exception
   */
  protected function getFixtureString(int $delta) {
    if (!empty($this->fixtures[$delta])) {
      $path_to_fixture = sprintf('%s/tests/fixtures/import/%s',
        drupal_get_path('module', 'acquia_contenthub'),
        $this->fixtures[$delta]['cdf']
      );
      return file_get_contents($path_to_fixture);
    }

    throw new \Exception(sprintf("Missing fixture for delta %d in class %s", $delta, __CLASS__));
  }

  /**
   * Returns fixture expectations.
   *
   * @param int $delta
   *   Delta.
   *
   * @return mixed
   *   Expectations array.
   *
   * @throws \Exception
   */
  protected function getFixtureExpectations(int $delta) {
    if (!empty($this->fixtures[$delta])) {
      $path_to_fixture = sprintf('%s/tests/fixtures/import/%s',
        drupal_get_path('module', 'acquia_contenthub'),
        $this->fixtures[$delta]['expectations']
      );

      return include $path_to_fixture;
    }

    throw new \Exception(sprintf("Missing expectations for delta %d in class %s", $delta, __CLASS__));
  }

  /**
   * Creates CDF document from fixture.
   *
   * @param int $delta
   *   The delta of the filename to retrieve from the local fixtures property.
   *
   * @return \Acquia\ContentHubClient\CDFDocument
   *   CDF document.
   *
   * @throws \Exception
   */
  protected function createCdfDocumentFromFixture(int $delta): CDFDocument {
    $json = $this->getFixtureString($delta);
    $data = Json::decode($json);
    $document_parts = [];
    foreach ($data['entities'] as $entity) {
      $document_parts[] = $this->populateCDFObject($entity);
    }

    $cdf_document = new CDFDocument(...$document_parts);

    return $cdf_document;
  }

  /**
   * Populates CDF object from array.
   *
   * @param array $data
   *   CDFObject data.
   *
   * @return \Acquia\ContentHubClient\CDF\CDFObject
   *   Populated CDF object.
   *
   * @throws \Exception
   *
   * @see \Acquia\ContentHubClient\ContentHubClient::getEntities()
   */
  protected function populateCDFObject(array $data) {
    $event = new GetCDFTypeEvent($data);
    $subscriber = new DefaultCDF();
    $subscriber->onGetCDFType($event);
    return $event->getObject();
  }

  /**
   * Import fixture.
   *
   * @param int $delta
   *   Delta.
   *
   * @return mixed
   *   Exptectations.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function importFixture(int $delta) {
    $expectations = $this->getFixtureExpectations($delta);
    $document = $this->createCdfDocumentFromFixture($delta);
    $stack = new DependencyStack();
    $this->getSerializer()->unserializeEntities($document, $stack);
    return $expectations;
  }

  /**
   * Import and export content.
   *
   * @param int $delta
   *   Fixture delta.
   * @param array $validate_data
   *   Data.
   * @param string $export_type
   *   Exported entity type.
   * @param string $export_uuid
   *   Entity UUID.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   * @throws \Exception
   */
  public function contentEntityImportExport($delta, $validate_data, $export_type, $export_uuid) {
    $expectations = $this->importFixture($delta);
    /** @var \Drupal\Core\Entity\EntityRepositoryInterface $repository */
    $repository = $this->container->get('entity.repository');
    foreach ($validate_data as $datum) {
      $entity_type = $datum['type'];
      $validate_uuid = $datum['uuid'];
      /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
      $entity = $repository->loadEntityByUuid($entity_type, $validate_uuid);
      if (!isset($expectations[$validate_uuid])) {
        throw new \Exception(sprintf("You are missing validation for the entity of type %s of uuid %s.", $entity_type, $validate_uuid));
      }
      $expectation = $expectations[$validate_uuid];
      foreach ($entity->getTranslationLanguages() as $language) {
        $trans = $entity->getTranslation($language->getId());
        /**
         * @var  $field_name
         * @var \Drupal\Core\Field\FieldItemListInterface $field
         */
        foreach ($trans as $field_name => $field) {
          if ($expectation->excludeField($field_name)) {
            continue;
          }
          //        print "'$field_name' => [\n  '{$language->getId()}' => ";
          //        print var_export($field->getValue(), TRUE) . "\n";
          //        print "  ],\n";
          $values = $this->handleFieldValues($field);
          $this->assertEquals($expectation->getFieldValue($field_name, $language->getId()), $values);
        }
      }
    }
    $export_entity = $repository->loadEntityByUuid($export_type, $export_uuid);
    $wrapper = new DependentEntityWrapper($export_entity);
    $stack = new DependencyStack();
    $this->getCalculator()->calculateDependencies($wrapper, $stack);
    $entities = NestedArray::mergeDeep([$wrapper->getUuid() => $wrapper], $stack->getDependenciesByUuid(array_keys($wrapper->getDependencies())));
    $data = $this->getSerializer()->serializeEntities(...array_values($entities));
    $document = new CDFDocument(...$data);
    $fixtures = json_decode($this->getFixtureString($delta), TRUE);
    /** @var \Acquia\ContentHubClient\CDF\CDFObject[] $objects */
    $objects = [];
    // Reindex objects for easier assertions.
    foreach ($document->getEntities() as $object) {
      if ($object->getType() !== 'drupal8_content_entity') {
        continue;
      }
      $objects[$object->getUuid()] = $object;
    }
    // Exclusively check content entities because configuration will need
    // separate test coverage.
    $count = 0;
    foreach ($fixtures['entities'] as $fixture) {
      if ($fixture['type'] !== 'drupal8_content_entity') {
        continue;
      }
      $count++;
      $object = json_decode(base64_decode($objects[$fixture['uuid']]->getMetadata()['data']), TRUE);
      $fixture = json_decode(base64_decode($fixture['metadata']['data']), TRUE);
      list($fixture, $object) = $this->normalizeFixtureAndObject($fixture, $object);
      $this->assertEquals($fixture, $object);
    }
    $this->assertEquals($count, count($objects));
  }

  /**
   * Get the CDF serializer.
   *
   * @return \Drupal\acquia_contenthub\EntityCdfSerializer
   *   CDF serializer.
   *
   * @throws \Exception
   */
  protected function getSerializer() {
    return \Drupal::service('entity.cdf.serializer');
  }

  /**
   * Get the dependency calculator.
   *
   * @return \Drupal\depcalc\DependencyCalculator
   *   Dependency calculator.
   *
   * @throws \Exception
   */
  protected function getCalculator() {
    return \Drupal::service('entity.dependency.calculator');
  }

  /**
   * Handle custom field types to more accurately match expectations.
   *
   * @param \Drupal\Core\Field\FieldItemListInterface $field
   *   The field being handled.
   *
   * @return array|mixed
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function handleFieldValues(FieldItemListInterface $field) {
    $values = $field->getValue();
    if (in_array($field->getFieldDefinition()->getType(), self::ENTITY_REFERENCE_TYPES) && $values) {
      $values = [];
      foreach ($field as $item_delta => $item) {
        if ($item->getValue()['target_id']) {
          $values[$item_delta]['target_id'] = $item->entity->uuid();
        }
      }
    }
    if ($field->getFieldDefinition()->getType() === self::ENTITY_REFERENCE_IMAGE_TYPE && $values) {
      $values = [];
      foreach ($field as $item_delta => $item) {
        if ($item->getValue()['target_id']) {
          $values[$item_delta] = $item->getValue();
          $values[$item_delta]['target_id'] = $item->entity->uuid();
        }
      }
    }
    if ($field->getFieldDefinition()->getType() === 'link') {
      foreach ($field as $item_delta => $item) {
        list($uri_type, $uri) = explode(':', $item->getValue()['uri']);
        if ($uri_type === 'entity') {
          list($item_entity_type, $item_entity_id) = explode('/', $uri);
          $uri_entity = $this->entityManager->getStorage($item_entity_type)->load($item_entity_id);
          $values[$item_delta]['uri'] = $uri_entity->uuid();
        }
        else {
          $values[$item_delta]['uri'] = $item->getValue()['uri'];
        }
      }
    }
    return $values;
  }

  /**
   * Normalize fixture and expected object.
   *
   * @param array $fixture
   *   Fixture.
   * @param array $object
   *   Object.
   *
   * @return array
   *   Normalized data.
   */
  protected function normalizeFixtureAndObject(array $fixture, array $object): array {
    $list = [
      'content_translation_changed',
      'content_translation_outdated',
      'content_translation_source',
      'menu_link',
      'revision_created',
      'revision_translation_affected',
    ];

    foreach ($list as $item) {
      if (isset($fixture[$item])) {
        unset($fixture[$item]);
      }

      if (isset($object[$item])) {
        unset($object[$item]);
      }
    }

    return [$fixture, $object];
  }

}
